unit ConnectFour;

{ Connect 4 1991 Rhys Hollow}
{This Rulebook and its source are both PovertyWare}

interface

	uses
		GameTypes, DialogSubs, GraphSubs, ConnectTypes, Draw4;

	procedure Main (var gameevent: gameEventRecord);

implementation


	procedure Main (var gameevent: gameEventRecord);

		var
			gglobals: globalsHandle;
			ggame: gamePeek;

		procedure MyDebug (s: string; n: longint);
			var
				numstr: str255;
		begin
			NumToString(n, numstr);
			DebugStr(concat(s, numstr));
		end;

		procedure InitGame;
			var
				i: integer;
		begin
			gameevent.globals := NewHandle(sizeof(globalsRecord));
			HLock(gameevent.globals);
			with globalsHandle(gameevent.globals)^^ do begin
				gamename := gameevent.message;
				for i := 0 to 7 do begin
					myblack[i] := 255;
				end;
			end; {with}
			gameevent.int1 := windwidth;
			gameevent.int2 := windheight;
		end; {proc}

		procedure FinishGame;
		begin
			DisposHandle(gameevent.globals);
			gameevent.globals := nil;
		end;

		procedure UpdateMenus;
		begin
		end;

		procedure UpdateControls;
			var
				i: integer;
				cnt: integer;
		begin
			cnt := 0;
			for i := 0 to xmax do begin
				GetRect(i, -1, ggame^.controls[i].cframe);
				if ggame^.playstate = ps_Won then
					ggame^.controls[i].piece := pt_None
				else if ggame^.board[i, 0] = pt_None then
					ggame^.controls[i].piece := ggame^.move
				else begin
					ggame^.controls[i].piece := pt_None;
					cnt := cnt + 1;
				end;
			end; {for}
			if cnt = xmax + 1 then
				ggame^.playstate := ps_Draw;
		end;

		procedure CommonInit;
			var
				savport: GrafPtr;
		begin
			GetPort(savport);
			SetDHandle(savport, useritemID, Handle(@Draw));
			WindowPeek(savport)^.refcon := longint(gameevent.game);
			ggame^.globals := gglobals;
		end;

		procedure GameNew;
			var
				tmpstr: str63;
				x: boardXRange;
				y: boardYRange;
				hstate: SignedByte;
		begin
			SetHandleSize(Handle(gameevent.game), sizeof(gameRecord));
			hstate := HGetState(Handle(gameevent.game));
			HLock(Handle(gameevent.game));
			ggame := gamePeek(Handle(gameevent.game)^);
			CommonInit;
			with ggame^ do begin
				for x := -1 to sentxmax do begin
					for y := -1 to sentymax do begin
						board[x, y] := pt_None;
					end; {for}
				end; {for}
				connectionstate := cs_Local;
				playstate := ps_Playing;
				sequence := 1;
				mypiece := pt_White;
				move := pt_White;
				gameevent.myturn := true;
				UpdateControls;
			end; {with}
		end;

		procedure OldGame;
		begin
			CommonInit;
			ggame^.connectionstate := cs_Local;
			gameevent.myturn := true;
			UpdateControls;
		end;

		function Opposite (p: pieceType): pieceType;
		begin
			case p of
				pt_White: 
					Opposite := pt_Black;
				pt_Black: 
					Opposite := pt_White;
				pt_None: 
					Opposite := pt_None;
				otherwise
			end;
		end;

		procedure Swap;
		begin
			with ggame^ do begin
				mypiece := Opposite(mypiece);
				gameevent.myturn := (mypiece = move) or (connectionstate = cs_Local);
			end; {with}
			UpdateControls;
			DrawControls(ggame);
			DrawScore(ggame);
		end;

		procedure ConnectionLost;
		begin
			with ggame^ do begin
				connectionstate := cs_Local;
				gameevent.myturn := true;
			end; {with}
			DrawScore(ggame);
		end;

		procedure ConnectionMade;
		begin
			with ggame^ do begin
				connectionstate := cs_Remote;
				gameevent.myturn := move = mypiece;
			end; {with}
			DrawScore(ggame);
		end;

		function TrackPt (x: integer): boolean;
			var
				r: rect;
				izin: boolean;
				pt: point;
		begin
			GetRect(x, -1, r);
			r.bottom := boardy + boarddeltay;
			DrawPiece(ggame, Opposite(ggame^.move), x, -1);
			izin := true;
			while StillDown do begin
				GetMouse(pt);
				if PtInRect(pt, r) then begin
					if not izin then begin
						izin := true;
						DrawPiece(ggame, Opposite(ggame^.move), x, -1);
					end;
				end
				else if izin then begin
					izin := false;
					DrawPiece(ggame, ggame^.move, x, -1);
				end;
			end;
			if izin then
				DrawPiece(ggame, ggame^.move, x, -1);
			TrackPt := izin;
		end;

		procedure NextPlayer;
		begin
			ggame^.move := Opposite(ggame^.move);
			gameevent.myturn := (ggame^.move = ggame^.mypiece) or (ggame^.connectionstate = cs_Local);
		end;

		function CheckLine (x, y, dx, dy: integer; hilite: boolean): boolean;
			var
				tx, ty: integer;
				pc: integer;
				tr: rect;
		begin
			pc := 1;
			ty := y + dy;
			tx := x + dx;
			while (pc < 4) and (ggame^.board[tx, ty] = ggame^.move) do begin
				if hilite then begin
					GetRect(tx, ty, tr);
					InsetRect(tr, -1, -1);
					InvertRect(tr);
				end; {if}
				pc := pc + 1;
				tx := tx + dx;
				ty := ty + dy;
			end; {while}
			if pc < 4 then begin
				tx := x - dx;
				ty := y - dy;
				while (pc < 4) and (ggame^.board[tx, ty] = ggame^.move) do begin
					if hilite then begin
						GetRect(tx, ty, tr);
						InsetRect(tr, -1, -1);
						InvertRect(tr);
					end; {if}
					pc := pc + 1;
					tx := tx - dx;
					ty := ty - dy;
				end; {while}
			end;  {if}
			if (pc >= 4) and hilite then begin
				GetRect(x, y, tr);
				InsetRect(tr, -1, -1);
				InvertRect(tr);
			end;
			CheckLine := (pc >= 4);
		end;

		function CheckWin (x, y: integer; hilite: boolean): boolean;
			var
				won: boolean;
				dx, dy: integer;
				dummy: boolean;
		begin
			won := false;
			dummy := CheckLine(x, y, -1, 0, false);
			won := won or dummy;
			if hilite and dummy then
				dummy := CheckLine(x, y, -1, 0, true);
			for dx := -1 to 1 do begin
				dummy := CheckLine(x, y, dx, -1, false);
				won := won or dummy;
				if hilite and dummy then
					dummy := CheckLine(x, y, dx, -1, true);
			end;
			CheckWin := won;
		end;

		procedure DropPiece (p: pieceType; x: boardXRange);
			var
				i, j: integer;
				f: longint;
				d: longint;
				won, dummy: boolean;
				tmpstr: str255;
		begin
			DrawPiece(ggame, p, x, 0);
			i := 1;
			while (i < sentymax) and (ggame^.board[x, i] = pt_None) do begin
				Delay(2, d);
				DrawPiece(ggame, pt_None, x, i - 1);
				DrawPiece(ggame, p, x, i);
				i := i + 1;
			end;
			ggame^.board[x, i - 1] := p;
			if (ggame^.connectionstate = cs_Remote) and (ggame^.move = ggame^.mypiece) then begin
				gameevent.event := ge_SendMessage;
				NumToString(ggame^.sequence, tmpstr);
				NumToString(x, gameevent.message);
				gameevent.message := concat(tmpstr, gameevent.message);
			end;
			won := CheckWin(x, i - 1, false);
			if won then begin
				for j := 1 to 8 do begin
					dummy := CheckWin(x, i - 1, true);
					Delay(5, f);
				end; {for}
				ggame^.playstate := ps_Won;
			end;
			gameevent.modified := true;
			NextPlayer;
			UpdateControls;
			DrawControls(ggame);
			DrawScore(ggame);
		end;

		procedure Restart;
			var
				i, j: integer;
				r: rect;
		begin
			ggame^.playstate := ps_Playing;
			for i := 0 to xmax do
				for j := 0 to ymax do
					ggame^.board[i, j] := pt_None;
			ggame^.sequence := ggame^.sequence + 1;
			gameevent.modified := true;
			UpdateControls;
			r.top := 0;
			r.left := 0;
			r.bottom := windheight;
			r.right := windwidth;
			InvalRect(r);
		end;

		procedure MouseDown;
			var
				r: rect;
				i: integer;
				j: integer;
		begin
			j := -1;
			for i := 0 to xmax do begin
				GetRect(i, -1, r);
				r.bottom := boardy + boarddeltay;
				if PtInRect(gameevent.where, r) and (ggame^.controls[i].piece <> pt_None) then
					j := i;
			end; {for}
			if j <> -1 then begin
				if TrackPt(j) then begin
					DropPiece(ggame^.move, j);
				end;
			end;
		end;

		procedure MessageReceived;
			var
				tmpstr: str255;
				x: longint;
				seq: longint;
				i, j: integer;
		begin
			tmpstr := gameevent.message;
			StringToNum(Copy(tmpstr, 1, 1), seq);
			StringToNum(Copy(tmpstr, 2, 1), x);
			if seq = ggame^.sequence then
				DropPiece(ggame^.move, x);
		end;

		var
			hstate: SignedByte;

	begin
		if (gameevent.game <> nil) and (gameevent.event <> ge_NewGame) then begin
			hstate := HGetState(Handle(gameevent.game));
			HLock(Handle(gameevent.game));
			ggame := gamePeek(Handle(gameevent.game)^);
		end;
		if gameevent.globals <> nil then begin
			HLock(gameevent.globals);
			gglobals := globalsHandle(gameevent.globals);
		end;
		case gameevent.event of
			ge_InitRuleBook: 
				InitGame;
			ge_FinishRuleBook: 
				FinishGame;
			ge_UpdateMenus: 
				UpdateMenus;
			ge_NewGame: 
				GameNew;
			ge_OldGame: 
				OldGame;
			ge_ConnectionLost: 
				ConnectionLost;
			ge_ConnectionMade: 
				ConnectionMade;
			ge_MessageReceived: 
				MessageReceived;
			ge_MouseDown: 
				MouseDown;
			ge_Swap: 
				Swap;
			ge_Restart: 
				Restart;
			otherwise
		end;
		if (gameevent.game <> nil) and (gameevent.event <> ge_NewGame) then
			HSetState(Handle(gameevent.game), hstate);
		if gameevent.globals <> nil then
			HUnlock(gameevent.globals);
	end;
end.