unit DerfBall;

{ DerfBall  Peter Lewis, Oct 1991 }
{ This program and its source are (is? who knows) Povertyware }

interface

	uses
		QuickDrawRules, GameTypes, FixMath, DerfTypes, DerfDraws;

	procedure Main (var ger: gameEventRecord);

implementation

	procedure Main (var ger: gameEventRecord);
		var
			gglobals: globalsPeek;
			ggame: gamePeek;
			gwindow: windowPtr;
			ghandle: handle;

		function GetGlobalString (i: integer): str255;
			var
				s: str255;
		begin
			GetIndString(s, 1000, i);
			GetGlobalString := s;
		end;

		procedure SetMyTurn;
		begin
			ger.myturn := (ggame^.playstate = ps_Playing);
		end;

		procedure UpdateControls;
		begin
			UpdateStatus(ggame);
		end;

		procedure ScrollRectNil (r: rect; dh, dv: integer);
			var
				rgn: RgnHandle;
				i: integer;
		begin
			rgn := NewRgn;
			BackPat(gglobals^.black);
			for i := 1 to slide_count do
				ScrollRect(r, dh div slide_count, dv div slide_count, rgn);
			BackPat(gglobals^.white);
			DisposeRgn(rgn);
		end;

		procedure SetBallInfo (b: integer);
			var
				bt: squareType;
				n: integer;
				topbot: integer;
		begin
			with ggame^ do begin
				bt := board[ball[b].row, ball[b].col];
				topbot := 2 * ord((ball[b].side = st_top) or (ball[b].side = st_bottom));
				case bt of
					st1, st6, st7: 
						n := 3 - topbot;
					st2, st5, st8: 
						n := 2;
					st3, st4, st9: 
						n := 1 + topbot;
				end;
				ball[b].out := sideType((ord(ball[b].side) + n) mod 4);
				ball[b].curved := not (bt in [st2, st5, st8]);
				if ball[b].curved then
					ball[b].max := curved_moves
				else
					ball[b].max := straight_moves;
				if not ball[b].curved then begin
					ball[b].dh1 := 0;
					ball[b].dv1 := 0;
					case ball[b].side of
						st_top: 
							ball[b].dv1 := 1;
						st_right: 
							ball[b].dh1 := -1;
						st_bottom: 
							ball[b].dv1 := -1;
						st_left: 
							ball[b].dh1 := 1;
					end;
				end
				else begin
					case ball[b].side of
						st_top:  begin
							ball[b].dh1 := 0;
							ball[b].dv1 := 1;
							ball[b].dv2 := 0;
							if (bt = st1) or (bt = st7) then
								ball[b].dh2 := 1
							else
								ball[b].dh2 := -1;
						end;
						st_right:  begin
							ball[b].dh1 := -1;
							ball[b].dv1 := 0;
							ball[b].dh2 := 0;
							if (bt = st1) or (bt = st7) then
								ball[b].dv2 := -1
							else
								ball[b].dv2 := 1;
						end;
						st_bottom:  begin
							ball[b].dh1 := 0;
							ball[b].dv1 := -1;
							ball[b].dv2 := 0;
							if (bt = st4) or (bt = st9) then
								ball[b].dh2 := 1
							else
								ball[b].dh2 := -1;
						end;
						st_left:  begin
							ball[b].dh1 := 1;
							ball[b].dv1 := 0;
							ball[b].dh2 := 0;
							if (bt = st3) or (bt = st9) then
								ball[b].dv2 := -1
							else
								ball[b].dv2 := 1;
						end;
					end;
				end;
			end;
		end;

		procedure GetSideOffset (s: sideType; var dr, dc: integer);
		begin
			dr := 0;
			dc := 0;
			case s of
				st_top: 
					dr := -1;
				st_right: 
					dc := 1;
				st_bottom: 
					dr := 1;
				st_left: 
					dc := -1;
			end;
		end;

		procedure GetAdjacent (r: rows; c: cols; s: sideType; var rr: rows; var cc: cols);
			var
				dr, dc: integer;
		begin
			GetSideOffset(s, dr, dc);
			rr := (r + dr + row_max - 1) mod row_max + 1;
			cc := (c + dc + col_max - 1) mod col_max + 1;
		end;

		procedure GetNextSquare (r: rows; c: cols; s: sideType; var rr, cc: integer);
			var
				dr, dc: integer;
		begin
			GetSideOffset(s, dr, dc);
			rr := r + dr;
			cc := c + dc;
		end;

		procedure DrawBall (b: integer);
			var
				brct: rect;
		begin
			SetBridgeClip(ggame, b);
			with ggame^.ball[b].pos do
				GetBallRect(h, v, brct);
			PaintOval(brct);
			ClipRect(gwindow^.portRect);
		end;

		procedure CheckBall (r1, c1, r2, c2: integer);
			procedure Fix (var a1, a2: integer);
				var
					a: integer;
			begin
				if a1 > a2 then begin
					a := a1;
					a1 := a2;
					a2 := a;
				end;
			end;
			var
				rr, cc: integer;
				brct: rect;
				b: integer;
		begin
			Fix(r1, r2);
			Fix(c1, c2);
			with ggame^ do
				for b := 1 to ball_max do
					if (r1 - 1 <= ball[b].row) and (ball[b].row <= r2 + 1) and (c1 - 1 <= ball[b].col) and (ball[b].col <= c2 + 1) then begin
						DrawBox(ggame, ball[b].row, ball[b].col);
						if ball[b].offset < 3 then begin
							GetNextSquare(ball[b].row, ball[b].col, ball[b].side, rr, cc);
							DrawBox0(ggame, rr, cc);
						end;
						if ball[b].offset > ball[b].max - 3 then begin
							GetNextSquare(ball[b].row, ball[b].col, ball[b].out, rr, cc);
							DrawBox0(ggame, rr, cc);
						end;
					end;
		end;

		procedure SlideRow (c: cols);
			var
				spc, rct: rect;
				i, b: integer;
		begin
			with ggame^, space[ord(c > col_mid)] do begin
				Checkball(row, col, row, c);
				GetBoxRect(ggame, spc, row, col);
				GetBoxRect(ggame, rct, row, c);
				if c > col then begin
					spc.right := rct.right;
					ScrollRectNil(spc, -square_width, 0);
					for i := col to c - 1 do
						board[row, i] := board[row, i + 1];
					for b := 1 to ball_max do
						if (col < ball[b].col) and (ball[b].col <= c) and (ball[b].row = row) then begin
							ball[b].col := ball[b].col - 1;
							ball[b].pos.h := ball[b].pos.h - square_width;
						end;
				end
				else begin
					spc.left := rct.left;
					ScrollRectNil(spc, square_width, 0);
					for i := col downto c + 1 do
						board[row, i] := board[row, i - 1];
					for b := 1 to ball_max do
						if (c <= ball[b].col) and (ball[b].col < col) and (ball[b].row = row) then begin
							ball[b].col := ball[b].col + 1;
							ball[b].pos.h := ball[b].pos.h + square_width;
						end;
				end;
				col := c;
				board[row, col] := st_None;
				for b := 1 to ball_max do
					DrawBall(b);
			end;
		end;

		procedure SlideCol (r: rows; theside: integer);
			var
				spc, rct: rect;
				i, b: integer;
		begin
			with ggame^, space[theside] do begin
				Checkball(row, col, r, col);
				GetBoxRect(ggame, spc, row, col);
				GetBoxRect(ggame, rct, r, col);
				if r > row then begin
					spc.bottom := rct.bottom;
					ScrollRectNil(spc, 0, -square_width);
					for i := row to r - 1 do
						board[i, col] := board[i + 1, col];
					for b := 1 to ball_max do
						if (row < ball[b].row) and (ball[b].row <= r) and (ball[b].col = col) then begin
							ball[b].row := ball[b].row - 1;
							ball[b].pos.v := ball[b].pos.v - square_height;
						end;
				end
				else begin
					spc.top := rct.top;
					ScrollRectNil(spc, 0, square_width);
					for i := row downto r + 1 do
						board[i, col] := board[i - 1, col];
					for b := 1 to ball_max do
						if (r <= ball[b].row) and (ball[b].row < row) and (ball[b].col = col) then begin
							ball[b].row := ball[b].row + 1;
							ball[b].pos.v := ball[b].pos.v + square_height;
						end;
				end;
				row := r;
				board[row, col] := st_None;
				for b := 1 to ball_max do
					DrawBall(b);
			end;
		end;

		procedure SendMove (ch: char; i: integer);
		begin
			if ggame^.connectionState = cs_remote then begin
				ger.message := concat(ch, chr(i + 32));
				ger.event := ge_SendMessage;
			end;
		end;

		procedure SetBallPos (b: integer);
			var
				srct: rect;
		begin
			with ggame^, globals^ do begin
				GetBoxRect(ggame, srct, ball[b].row, ball[b].col);
				ball[b].pos.h := srct.left + sideh[ball[b].side];
				ball[b].pos.v := srct.top + sidev[ball[b].side];
			end;
		end;

		function NumToStr (n: longInt): str255;
			var
				s: str255;
		begin
			NumToString(n, s);
			NumToStr := s;
		end;

		procedure NewBall (b: integer; theside: boolean);
			function ValidSide (side: sideType; bt: squareType): boolean;
			begin
				case side of
					st_top: 
						ValidSide := bt in [st1, st2, st3, st7, st8, st9];
					st_right: 
						ValidSide := bt in [st1, st4, st5, st7, st8, st9];
					st_bottom: 
						ValidSide := bt in [st2, st4, st6, st7, st8, st9];
					st_left: 
						ValidSide := bt in [st3, st5, st6, st7, st8, st9];
				end;
			end;
		begin
			with ggame^, ball[b] do begin
				repeat
					row := Rand(rnd1, rnd2, row_max) + 1;
					col := Rand(rnd1, rnd2, col_mid - 2) + 2;
					if theside then
						col := col + col_mid;
				until not (board[row, col] in [st_None, st_Empty]);
				offset := 0;
				time := TickCount;
				repeat
					side := sideType(Rand(rnd1, rnd2, 4));
{    DebugStr(concat('Row=', NumToStr(row), ', Col=', NumToStr(col), ', Side=', NumToStr(ord(side)), ', Board=', NumToStr(ord(board[row, col])), ', Valid=', NumToStr(ord(ValidSide(side, board[row, col])))));}
				until ValidSide(side, board[row, col]);
				SetBallPos(b);
				SetBallInfo(b);
			end;
		end;

		procedure RemoveBall (b: integer);
			var
				brct, srct: rect;
				rr, cc: integer;
		begin
			with ggame^ do begin
				GetBallRect(ball[b].pos.h, ball[b].pos.v, brct);
				if ball[b].offset <= 2 then begin
					GetBoxRect(ggame, srct, ball[b].row, ball[b].col);
					ClipRect(srct);
					EraseOval(brct);
					ClipRect(gwindow^.portRect);
					GetNextSquare(ball[b].row, ball[b].col, ball[b].side, rr, cc);
					DrawBox0(ggame, rr, cc);
				end
				else if ball[b].offset >= ball[b].max - 2 then begin
					DrawBox(ggame, ball[b].row, ball[b].col);
					GetNextSquare(ball[b].row, ball[b].col, ball[b].out, rr, cc);
					DrawBox0(ggame, rr, cc);
				end
				else begin
					SetBridgeClip(ggame, b);
					EraseOval(brct);
				end;
			end;
		end;

		procedure KillBall (b: integer; ch: char; send: boolean);
		begin
			with ggame^, ball[b] do begin
				RemoveBall(b);
				ballcount[col > col_mid] := ballcount[col > col_mid] - 1;
				DrawStatus(ggame);
				if (ballcount[col > col_mid] > 0) and send then begin
					NewBall(b, col <= col_mid);
					ger.message := concat(ch, chr(b + 32), chr(row + 32), chr(col + 32), chr(ord(side) + 32));
					ger.event := ge_SendMessage;
				end;
				if (ballcount[col > col_mid] = 0) then begin
					ggame^.playState := ps_won;
					DrawDeadBall(ggame, b);
				end;
			end;
		end;

		procedure DoRestart;
			var
				r: rows0;
				c: cols0;
				i: integer;
				bool: boolean;
				b: integer;
		begin
			with ggame^ do begin
				for r := 1 to row_max do
					for c := 1 to col_max do
						board[r, c] := squareType(Rand(rnd1, rnd2, 3) + 8);
				for r := 1 to row_max do begin
					board[r, 1] := st5;
					board[r, col_mid] := st5;
					board[r, col_mid + 1] := st5;
					board[r, col_max] := st5;
				end;
				for i := 0 to 1 do
					with space[i] do begin
						row := row_max div 2;
						col := col_mid * i + col_mid div 2;
						board[row, col] := st_None;
					end;
				for b := 1 to ball_max do
					NewBall(b, odd(b));
				for bool := false to true do begin
					wrapcount[bool] := initial_wrapcount;
					squarecount[bool] := 0;
					ballcount[bool] := initial_balls;
				end;
				DrawStatus(ggame);
				playstate := ps_Playing;
				ger.modified := false;
				SetMyTurn;
			end; {with}
		end;

		procedure RedrawAll;
		begin
			EraseRect(gglobals^.full_rect);
			SetPort(gwindow);
			DrawGame(gwindow, my_dialog_item);
		end;

		procedure SwapSides;
			var
				r: rows;
				c: cols;
				bt: squareType;
				b: integer;
		begin
			with ggame^ do begin
				for r := 1 to row_max do
					for c := 2 to col_mid - 1 do begin
						bt := board[r, c];
						board[r, c] := board[r, c + col_mid];
						board[r, c + col_mid] := bt;
					end;
				r := space[0].row;
				space[0].row := space[1].row;
				space[1].row := r;
				c := space[0].col;
				space[0].col := space[1].col - col_mid;
				space[1].col := c + col_mid;
				for b := 1 to ball_max do begin
					with ball[b] do
						if col > col_mid then
							col := col - col_mid
						else
							col := col + col_mid;
					master := not master;
					SetBallPos(b);
				end;
			end;
			SetMyTurn;
			RedrawAll;
		end;

		procedure ZotBallPipe (b: integer);
			procedure Convert (var bt: squareType);
			begin
				with ggame^.ball[b] do
					case bt of
						st1..st6: 
							bt := st_Empty;
						st7: 
							if (side = st_top) or (side = st_right) then
								bt := st6
							else
								bt := st1;
						st8: 
							if (side = st_top) or (side = st_bottom) then
								bt := st5
							else
								bt := st2;
						st9: 
							if (side = st_top) or (side = st_left) then
								bt := st4
							else
								bt := st3;
					end;
			end;
			var
				bt: squareType;
				rr, cc: integer;
				wr, bb: integer;
		begin
			with ggame^ do begin
				bt := board[ball[b].row, ball[b].col];
				if (ball[b].col > 1) and (ball[b].col <> col_mid) and (ball[b].col <> col_mid + 1) and (ball[b].col < col_max) then begin
					Convert(bt);
					for bb := 1 to ball_max do
						if b <> bb then begin
							if ((ball[b].side = ball[bb].side) or (ball[b].out = ball[bb].side)) and (ball[b].row = ball[bb].row) and (ball[b].col = ball[bb].col) then
								KillBall(bb, 'K', true);
						end;
					wr := wrapcount[ball[b].col > col_mid];
					if wr = 1 then begin
						wrapcount[ball[b].col > col_mid] := 0;
						DrawOutline(ggame);
						UpdateStatus(ggame);
					end
					else if wr > 0 then begin
						wrapcount[ball[b].col > col_mid] := wr - 1;
						UpdateStatus(ggame);
					end;
					squarecount[ball[b].col > col_mid] := squarecount[ball[b].col > col_mid] + 1;
				end;
				board[ball[b].row, ball[b].col] := bt;
				DrawBox(ggame, ball[b].row, ball[b].col);
				GetNextSquare(ball[b].row, ball[b].col, ball[b].out, rr, cc);
				DrawBox0(ggame, rr, cc);
			end;
		end;

		procedure ReceiveMessage (s: str63);
			var
				brct: rect;
				b: integer;
		begin
			ger.modified := true;
			with ggame^ do
				case s[1] of
					'R': 
						SlideRow(ord(s[2]) - 32 + col_mid);
					'C': 
						SlideCol(ord(s[2]) - 32, 1);
					'X':  begin
						rnd1 := restartRecord(s).rnd1;
						rnd2 := restartRecord(s).rnd2;
						DoRestart;
						SwapSides;
					end;
					'B':  begin
						b := ord(s[2]) - 32;
						with ball[b] do begin
							ZotBallPipe(b);
							row := ord(s[3]) - 32;
							col := ord(s[4]) - 32;
							if col > col_mid then
								col := col - col_mid
							else
								col := col + col_mid;
							side := sideType(ord(s[5]) - 32);
							offset := 0;
							time := TickCount;
							SetBallPos(b);
							SetBallInfo(b);
							DrawBall(b);
						end;
					end;
					'L', 'K':  begin
						b := ord(s[2]) - 32;
						with ball[b] do begin
							if s[1] = 'L' then
								ZotBallPipe(b);
							KillBall(b, 'x', false);
							row := ord(s[3]) - 32;
							col := ord(s[4]) - 32;
							if col > col_mid then
								col := col - col_mid
							else
								col := col + col_mid;
							side := sideType(ord(s[5]) - 32);
							offset := 0;
							time := TickCount;
							SetBallPos(b);
							SetBallInfo(b);
							ger.modified := false;
						end;
					end;
				end;
		end;

		procedure DoClick (where: point);
			var
				r, c, theside: integer;
		begin
			with ggame^, gglobals^ do
				if playstate = ps_playing then begin
					r := (where.v - item_rect.top) div square_height + 1;
					c := (where.h - item_rect.left) div square_width + 1;
					if (r >= 1) and (r <= row_max) and (c > 1) and (c < col_max) and (c <> col_mid) and (c <> col_mid + 1) then
						if (connectionState = cs_local) or (c < col_mid) then begin
							theside := ord(c > col_mid);
							with space[theside] do
								if (r = row) and (c <> col) then begin
									SlideRow(c);
									SendMove('R', c);
									ger.modified := true;
								end
								else if (r <> row) and (c = col) then begin
									SlideCol(r, theside);
									SendMove('C', r);
									ger.modified := true;
								end;
						end;
				end;
		end;

		procedure MoveBall (b: integer);
			var
				h, v: integer;
				r: rows;
				c: cols;
				rr, cc: integer;
				srct, brct, rct: rect;
				bt: squareType;
				lost: boolean;
		begin
			ger.modified := true;
			with ggame^, gglobals^ do
				if (ball[b].offset < ball[b].max) or (connectionState = cs_local) or (ball[b].col <= col_mid) then begin
					GetBoxRect(ggame, srct, ball[b].row, ball[b].col);
					h := srct.left + sideh[ball[b].side];
					v := srct.top + sidev[ball[b].side];
					if ball[b].curved then begin
						h := h + offc1[ball[b].offset] * ball[b].dh1 + offc2[ball[b].offset] * ball[b].dh2;
						v := v + offc1[ball[b].offset] * ball[b].dv1 + offc2[ball[b].offset] * ball[b].dv2;
					end
					else begin
						h := h + offs1[ball[b].offset] * ball[b].dh1 + offs2[ball[b].offset] * ball[b].dh2;
						v := v + offs1[ball[b].offset] * ball[b].dv1 + offs2[ball[b].offset] * ball[b].dv2;
					end;
					RemoveBall(b);
					ball[b].pos.h := h;
					ball[b].pos.v := v;
					DrawBall(b);
					if ball[b].max = ball[b].offset then begin
						ZotBallPipe(b);
						GetNextSquare(ball[b].row, ball[b].col, ball[b].out, rr, cc);
						GetAdjacent(ball[b].row, ball[b].col, ball[b].out, r, c);
						if (rr <> r) and (wrapcount[ball[b].col > col_mid] > 0) then begin
							lost := true;
						end
						else begin
							ball[b].row := r;
							ball[b].col := c;
							ball[b].side := sideType((ord(ball[b].out) + 2) mod 4);
							SetBallInfo(b);
							ball[b].offset := 1;
							bt := board[ball[b].row, ball[b].col];
							if (bt = st_None) or (bt = st_Empty) then
								lost := true
							else
								case ball[b].side of
									st_top: 
										lost := bt in [st4, st5, st6];
									st_right: 
										lost := bt in [st2, st3, st6];
									st_bottom: 
										lost := bt in [st1, st3, st5];
									st_left: 
										lost := bt in [st1, st2, st4];
								end;
						end;
						ger.event := ge_SendMessage;
						if lost then begin
							KillBall(b, 'L', true);
							ger.modified := false;
						end
						else begin
							SetBallPos(b);
							ger.message := concat('B', chr(b + 32), chr(ball[b].row + 32), chr(ball[b].col + 32), chr(ord(ball[b].side) + 32));
							DrawBall(b);
						end;
					end{if next square }
					else
						ball[b].offset := ball[b].offset + 1;
				end;{with}
		end;{proc}

		procedure DoGameIdle;
			var
				tmpspeed: integer;
				b: integer;
		begin
			with ggame^ do
				if playState = ps_playing then
					if ggame^.infront or (connectionState = cs_remote) then
						if not ger.dialoging then begin
							tmpspeed := speed;
							if BAND(ger.modifiers, shiftKey) <> 0 then
								tmpspeed := speed div 4;
							for b := 1 to ball_max do begin
								if TickCount > ball[b].time + tmpspeed then begin
									MoveBall(b);
									ball[b].time := ball[b].time + tmpspeed;
								end;
							end;
						end
						else begin
							for b := 1 to ball_max do begin
								DrawBall(b);
								ball[b].time := TickCount;
							end;
						end;
		end;

		procedure InitRuleBook;
			var
				i: integer;
				rct: rect;
				hdl: handle;
				bm: BitMap;
			procedure Move (off1, off2: integer);
			begin
				i := i + 1;
				with gglobals^ do begin
					offc1[i] := offc1[i - 1] + off1;
					offc2[i] := offc2[i - 1] + off2;
				end;
			end;
		begin
			ger.globals := Handle(NewPtr(sizeof(globalsRecord)));
			gglobals := globalsPeek(ger.globals);
			with rct, gglobals^ do begin
				for i := 0 to 3 do begin
					black[i * 2 + 0] := 255;
					black[i * 2 + 1] := 255;
					white[i * 2 + 0] := 0;
					white[i * 2 + 1] := 0;
					gray[i * 2 + 0] := $AA;
					gray[i * 2 + 1] := $55;
				end;
				for i := 0 to straight_moves do begin
					offs1[i] := i * 2;
					offs2[i] := 0;
				end;
				i := 0;
				offc1[i] := 0;
				offc2[i] := 0;
				Move(2, 0);
				Move(2, 1);
				Move(2, 1);
				Move(2, 1);
				Move(1, 1);
				Move(1, 2);
				Move(1, 2);
				Move(1, 2);
				Move(0, 2);
				sideh[st_top] := 12;
				sidev[st_top] := 0;
				sideh[st_right] := 24;
				sidev[st_right] := 12;
				sideh[st_bottom] := 12;
				sidev[st_bottom] := 24;
				sideh[st_left] := 0;
				sidev[st_left] := 12;
				square_port := GrafPtr(NewPtr(SizeOf(GrafPort)));
				hdl := Get1Resource('PICT', 1000);
				rct := PicHandle(hdl)^^.PicFrame;
				with rct do begin
					OffsetRect(rct, -left, -top);
					bm.rowbytes := (right + 15) div 16 * 2;
					bm.baseaddr := NewPtr(bm.rowbytes * (bottom));
					bm.bounds := rct;
				end;
				OpenPort(square_port);
				SetPortBits(bm);
				EraseRect(bm.bounds);
				DrawPicture(PicHandle(hdl), bm.bounds);
				ReleaseResource(hdl);

				rct.top := 0;
				rct.bottom := row_max * square_height;
				rct.left := 0;
				rct.right := col_max * square_width;
				OffsetRect(rct, insetH, insetV);
				item_rect := rct;
				status_rect := rct;
				status_rect.top := rct.bottom + status_offset + width;
				status_rect.bottom := status_rect.top + status_height;

				full_rect := rct;
				full_rect.bottom := status_rect.bottom;
				InsetRect(full_rect, -insetH, -insetV);

				ger.int1 := full_rect.right;
				ger.int2 := full_rect.bottom;
			end;
		end; {proc}

		procedure FinishRuleBook;
		begin
			with gglobals^ do begin
				DisposPtr(square_port^.portbits.baseaddr);
				ClosePort(square_port);
				DisposPtr(ptr(square_port));
			end;
			with ger do begin
				DisposPtr(POINTER(globals));
				globals := nil;
			end;
		end;

		procedure CommonInit;
			var
				k: integer;
				h: handle;
				r: rows0;
				c: cols0;
				rct: rect;
				b: integer;
		begin
			ggame^.globals := gglobals;
			GetDItem(gwindow, my_dialog_item, k, h, rct);
			rct := gglobals^.item_rect;
			InsetRect(rct, -3, -3);
			SetDItem(gwindow, my_dialog_item, k, handle(@DrawGame), rct);
			for b := 1 to ball_max do
				ggame^.ball[b].time := TickCount;
			SetWRefCon(gwindow, longInt(ghandle));
			with ggame^ do begin
				infront := false;
				connectionState := cs_local;
			end;
		end;

		procedure RestartGame;
			var
				s: str63;
		begin
			with ggame^ do
				if (master) or (connectionState = cs_local) then begin
					QDGlobals^.RandSeed := TickCount;
					rnd1 := BXOR(BSL(Random, 16), Random);
					rnd2 := 1;
					DoRestart;
					if connectionState = cs_remote then begin
						s := 'X11112222';
						restartRecord(s).rnd1 := rnd1;
						restartRecord(s).rnd2 := rnd2;
						ger.message := s;
						ger.event := ge_SendMessage;
					end;
				end;
		end;

		procedure NewGame;
			var
				r: rows0;
				c: cols0;
		begin
			HUnlock(ghandle);
			SetHandleSize(ghandle, SizeOf(gameRecord));
			HLock(ghandle);
			ggame := gamePeek(ghandle^);
			ggame^.master := true;
			CommonInit;
			with ggame^ do begin
				RestartGame;
			end; {with}
		end;

		procedure OldGame;
		begin
			CommonInit;
			SetMyTurn;
			UpdateControls;
		end;

		procedure Swap;
		begin
			with ggame^ do
				master := not master;
			SwapSides;
		end;

		procedure ConnectionLost;
		begin
			ggame^.connectionState := cs_local;
			ggame^.master := true;
			SetMyTurn;
		end;

		procedure ConnectionMade;
		begin
			ggame^.connectionState := cs_remote;
			SetMyTurn;
		end;

		procedure Restart;
			var
				r: rect;
		begin
			RestartGame;
			RedrawAll;
		end;

		procedure MouseDown;
		begin
			DoClick(ger.where);
		end;

		procedure MessageReceived;
		begin
			ReceiveMessage(ger.message);
		end;

		procedure SleepQuery;
		begin
			ger.int1 := 5;
		end;

		procedure DoActivate;
			var
				b: integer;
		begin
			ggame^.infront := true;
			if ggame^.connectionState = cs_local then
				for b := 1 to ball_max do
					ggame^.ball[b].time := TickCount;
			InvalRect(ggame^.globals^.full_rect);
		end;

		procedure DoDeactivate;
		begin
			ggame^.infront := false;
			InvalRect(ggame^.globals^.full_rect);
		end;

	begin
		gglobals := globalsPeek(ger.globals);
		ghandle := ger.game;
		if ghandle <> nil then begin
			HLock(ghandle);
			ggame := gamePeek(ghandle^);
		end;
		GetPort(gwindow);
		case ger.event of
			ge_InitRuleBook: 
				InitRuleBook;
			ge_FinishRuleBook: 
				FinishRuleBook;
			ge_NewGame: 
				NewGame;
			ge_OldGame: 
				OldGame;
			ge_ConnectionLost: 
				ConnectionLost;
			ge_ConnectionMade: 
				ConnectionMade;
			ge_MessageReceived: 
				MessageReceived;
			ge_MouseDown: 
				MouseDown;
			ge_SleepQuery: 
				SleepQuery;
			ge_Idle: 
				DoGameIdle;
			ge_Activate: 
				DoActivate;
			ge_Deactivate: 
				DoDeactivate;
			ge_Swap: 
				Swap;
			ge_Restart: 
				Restart;
			otherwise
		end;
		PenNormal;
		if ghandle <> nil then
			HUnlock(ghandle);
	end;

end.