unit DotsNBoxes;

{ Dots & Boxes  Peter Lewis, Oct 1991 }
{ This program and its source are (is? who knows) Povertyware }

interface

	uses
		GameTypes, FixMath;

	procedure Main (var ger: gameEventRecord);

implementation

	const
		my_dialog_item = 1;
		row_max = 11;
		col_max = 13;
		width = 2;
		inset = 2;
		box_width = 16;
		box_offset = 5;
		score_offset = 5;
		score_height = 20;
		score_base = 6;
		loose_dot_count = 4;

	type
		playStateType = (ps_Playing, ps_Won);
		connectionStateType = (cs_Local, cs_Remote);
		pieceType = (pt_None, pt_Black, pt_grey);
		rows0 = 0..row_max;
		cols0 = 0..col_max;
		rows = 1..row_max;
		cols = 1..col_max;
		linesArray = array[boolean, rows0, cols0] of boolean;
		boardArray = array[rows, cols] of pieceType;
		globalsRecord = record
				black: pattern;
				gray: pattern;
				white: pattern;
				player_rect, item_rect, winning_rect, score_rect: rect;
				square_size, score_pos: point;
			end;
		globalsPeek = ^globalsRecord;
		gameRecord = record
				globals: globalsPeek;
				boxes: array[pieceType] of integer;
				lines: linesArray;
				board: boardArray;
				mypiece: pieceType;
				move: pieceType;
				connectionstate: connectionStateType;
				playstate: playStateType;
				scoreStr: str63;
			end;
		gamePeek = ^gameRecord;

	const
		first_player = pt_Grey;

	procedure Fail (s: str255);
	begin
		DebugStr(s);
	end;

	procedure MyDebug (s: string; n: longint);
		var
			numstr: str255;
	begin
		NumToString(n, numstr);
		DebugStr(concat(s, numstr));
	end;

	function DotR (gglobals: globalsPeek; r: rows0): integer;
	begin
		with gglobals^.item_rect, gglobals^.square_size do
			DotR := top + width + inset + r * v;
	end;

	function DotC (gglobals: globalsPeek; c: cols0): integer;
	begin
		with gglobals^.item_rect, gglobals^.square_size do
			DotC := left + width + inset + c * h;
	end;

	function OwnerPattern (gglobals: globalsPeek; pat: pieceType): pattern;
	begin
		case pat of
			pt_None: 
				OwnerPattern := gglobals^.white;
			pt_Black: 
				OwnerPattern := gglobals^.black;
			pt_grey: 
				OwnerPattern := gglobals^.gray;
		end;
	end;

	procedure DrawBox (ggame: gamePeek; r: rows; c: cols);
		var
			rct: rect;
	begin
		rct.top := DotR(ggame^.globals, r - 1) + width;
		rct.bottom := DotR(ggame^.globals, r);
		rct.left := DotC(ggame^.globals, c - 1) + width;
		rct.right := DotC(ggame^.globals, c);
		FillRect(rct, OwnerPattern(ggame^.globals, ggame^.board[r, c]));
	end;

	procedure DrawPlayerBox (ggame: gamePeek);
		var
			r: rect;
	begin
		r := ggame^.globals^.player_rect;
		InsetRect(r, width, width);
		FrameRect(ggame^.globals^.player_rect);
		FillRect(r, OwnerPattern(ggame^.globals, ggame^.move));
	end;

	procedure DrawWinningBox (ggame: gamePeek);
		procedure FillBit (r: rect; v1, v2: integer; pat: pieceType);
		begin
			r.top := v1;
			r.bottom := v2;
			FillRect(r, OwnerPattern(ggame^.globals, pat));
		end;

		var
			r: rect;
			v0, v1, v2, v3, vv: integer;
	begin
		r := ggame^.globals^.winning_rect;
		InsetRect(r, width, width);
		v0 := r.top;
		v3 := r.bottom;
		v1 := v0 + FracMul(v3 - v0, FracDiv(ggame^.boxes[pt_Black], col_max * row_max));
		v2 := v3 - FracMul(v3 - v0, FracDiv(ggame^.boxes[pt_grey], col_max * row_max));
		vv := (v0 + v3) div 2;
		FrameRect(ggame^.globals^.winning_rect);
		FillBit(r, v0, v1, pt_Black);
		FillBit(r, v1, v2, pt_None);
		FillBit(r, v2, v3, pt_grey);
		MoveTo(r.left - width, vv);
		Line(-width, 0);
		MoveTo(r.right, vv);
		Line(width, 0)
	end;

	procedure DrawScore (ggame: gamePeek);
		var
			r: rect;
	begin
		r := ggame^.globals^.score_rect;
		FrameRect(r);
		InsetRect(r, width, width);
		EraseRect(r);
		with ggame^.globals^.score_pos do
			MoveTo(h - StringWidth(ggame^.scoreStr) div 2, v);
		DrawString(ggame^.scoreStr);
	end;

	procedure DrawLine (gglobals: globalsPeek; vert: boolean; r: rows0; c: cols0);
	begin
		if vert then begin
			MoveTo(DotC(gglobals, c), DotR(gglobals, r) - width);
			LineTo(DotC(gglobals, c), DotR(gglobals, r - 1) + width);
		end
		else begin
			MoveTo(DotC(gglobals, c) - width, DotR(gglobals, r));
			LineTo(DotC(gglobals, c - 1) + width, DotR(gglobals, r));
		end;
	end;

	procedure EraseLine (gglobals: globalsPeek; vert: boolean; r: rows0; c: cols0);
	begin
		PenMode(patBic);
		if vert then begin
			MoveTo(DotC(gglobals, c), DotR(gglobals, r) - width);
			LineTo(DotC(gglobals, c), DotR(gglobals, r - 1) + width);
		end
		else begin
			MoveTo(DotC(gglobals, c) - width, DotR(gglobals, r));
			LineTo(DotC(gglobals, c - 1) + width, DotR(gglobals, r));
		end;
		PenMode(patOr);
	end;

	function CountLines (ggame: gamePeek; r: rows0; c: cols0): integer;
		var
			cnt: integer;
	begin
		cnt := 0;
		with ggame^ do begin
			if c > 0 then
				cnt := cnt + ord(lines[false, r, c]);
			if c < col_max then
				cnt := cnt + ord(lines[false, r, c + 1]);
			if r > 0 then
				cnt := cnt + ord(lines[true, r, c]);
			if r < row_max then
				cnt := cnt + ord(lines[true, r + 1, c]);
		end;
		CountLines := cnt;
	end;

	procedure DrawDot (ggame: gamePeek; r: rows0; c: cols0);
	begin
		MoveTo(DotC(ggame^.globals, c), DotR(ggame^.globals, r));
		Line(0, 0);
	end;

	procedure DrawGame (wp: windowPtr; item: integer);
		var
			h: handle;
			ggame: gamePeek;
			r: rows0;
			c: cols0;
	begin
		h := handle(GetWRefCon(wp));
		HLock(h);
		ggame := gamePeek(h^);
		PenSize(width, width);
		FrameRect(ggame^.globals^.item_rect);
		DrawPlayerBox(ggame);
		DrawWinningBox(ggame);
		DrawScore(ggame);
		with ggame^ do
			for r := 0 to row_max do begin
				for c := 0 to col_max do begin
					if CountLines(ggame, r, c) < loose_dot_count then
						DrawDot(ggame, r, c);
					if r > 0 then
						if lines[true, r, c] then
							DrawLine(ggame^.globals, true, r, c);
					if c > 0 then begin
						if lines[false, r, c] then
							DrawLine(ggame^.globals, false, r, c);
						if r > 0 then begin
							DrawBox(ggame, r, c);
						end;
					end;
				end;
			end;
		PenNormal;
		HUnlock(h);
	end;

	procedure Main (var ger: gameEventRecord);
		var
			gglobals: globalsPeek;
			ggame: gamePeek;
			gwindow: windowPtr;
			ghandle: handle;

		function Opposite (p: pieceType): pieceType;
		begin
			case p of
				pt_Grey: 
					Opposite := pt_Black;
				pt_Black: 
					Opposite := pt_Grey;
				pt_None: 
					Opposite := pt_None;
				otherwise
			end;
		end;

		function FindLine (where: point; var r: rows0; var c: cols0; var vert: boolean): boolean;
			var
				hoff, voff, hpos, vpos, hp, vp: integer;
				fl, horiz: boolean;
		begin
			with gglobals^.item_rect, gglobals^.square_size do begin
				hpos := where.h - left - width - width div 2 - inset;
				vpos := where.v - top - width - width div 2 - inset;
				hoff := (hpos + h div 2) mod h - h div 2;
				voff := (vpos + v div 2) mod v - v div 2;
				vert := abs(hoff) <= h div 6;
				horiz := abs(voff) <= v div 6;
				fl := (vert or horiz) and not (vert and horiz);
				if fl then begin
					if vert then begin
						hp := (hpos + h div 2) div h;
						vp := vpos div v + 1;
						if (hp < 0) or (hp > col_max) or (vp < 1) or (vp > row_max) then
							fl := false;
					end
					else begin
						vp := (vpos + v div 2) div v;
						hp := hpos div h + 1;
						if (hp < 1) or (hp > col_max) or (vp < 0) or (vp > row_max) then
							fl := false;
					end;
				end;
{writeln(hp : 4, ',', hpos : 4, ',', hoff : 4, ',', vp : 4, ',', vpos : 4, ',', voff : 4, ',', fl, ',', vert);}
			end;
			if fl then begin
				r := vp;
				c := hp;
			end;
			FindLine := fl;
		end;

		function GetGlobalString (i: integer): str255;
			var
				s: str255;
		begin
			GetIndString(s, 1000, i);
			GetGlobalString := s;
		end;

		procedure SetMyTurn;
		begin
			ger.myturn := (ggame^.move = ggame^.mypiece) or (ggame^.connectionstate = cs_Local);
			with ggame^ do
				if playstate = ps_Playing then
					scoreStr := GetGlobalString(ord(connectionState = cs_local) * 4 + ord(move = pt_grey) * 2 + ord(mypiece = pt_grey) + 1);
			DrawScore(ggame);
		end;

		procedure NextPlayer;
		begin
			with ggame^ do begin
				move := Opposite(move);
				SetMyTurn;
				DrawPlayerBox(ggame);
			end;
		end;

		function CheckWin: pieceType;
			var
				w: pieceType;
		begin
			w := pt_none;
			if (ggame^.boxes[ggame^.move] > (row_max * col_max) div 2) then
				w := ggame^.move;
			CheckWin := w;
		end;

		procedure AddLine (vert: boolean; r: rows0; c: cols0);
			var
				gotone: boolean;
				winner: pieceType;
			procedure CheckBox (r, c: integer);
			begin
				if (1 <= r) and (r <= row_max) and (1 <= c) and (c <= col_max) then
					with ggame^ do
						if lines[true, r, c] and lines[true, r, c - 1] and lines[false, r, c] and lines[false, r - 1, c] then
							if board[r, c] = pt_None then begin
								board[r, c] := move;
								DrawBox(ggame, r, c);
								boxes[move] := boxes[move] + 1;
								gotone := true;
							end;
			end;
			procedure CheckDot (r: rows0; c: cols0);
			begin
				if CountLines(ggame, r, c) = loose_dot_count then begin
					PenMode(patBic);
					DrawDot(ggame, r, c);
					PenMode(patOr);
				end;
			end;
			var
				f: longInt;
				i: integer;
		begin
			for i := 1 to 3 do begin
				EraseLine(ggame^.globals, vert, r, c);
				Delay(3, f);
				DrawLine(ggame^.globals, vert, r, c);
				Delay(3, f);
			end;
			ggame^.lines[vert, r, c] := true;
			gotone := false;
			CheckBox(r, c);
			CheckDot(r, c);
			if vert then begin
				CheckDot(r - 1, c);
				CheckBox(r, c + 1)
			end
			else begin
				CheckDot(r, c - 1);
				CheckBox(r + 1, c);
			end;
			winner := CheckWin;
			if winner <> pt_none then
				with ggame^ do begin
					playState := ps_won;
					if connectionstate <> cs_local then
						scoreStr := GetGlobalString(ord(winner <> mypiece) + 9)
					else
						scoreStr := GetGlobalString(ord(winner = pt_grey) + 11);
					DrawScore(ggame);
				end
			else if not gotone then
				NextPlayer;
			if gotone then
				DrawWinningBox(ggame);
		end;

		procedure SendMove (vert: boolean; r: rows0; c: cols0);
			function NumToStr (n: integer): str15;
				var
					s: str255;
			begin
				NumToString(n, s);
				while length(s) < 3 do
					s := concat('0', s);
				NumToStr := s;
			end;
		begin
			ger.event := ge_SendMessage;
			ger.message := concat('M', chr(ord(vert) + 48), NumToStr(r), NumToStr(c));
		end;

		procedure DoMove (s: str15);
			function StrToNum (s: str15; off: integer): integer;
				var
					n: longInt;
			begin
				StringToNum(copy(s, off, 3), n);
				StrToNum := n;
			end;
		begin
			if length(s) <> 8 then
				Fail('Message not 8 chars')
			else if s[1] <> 'M' then
				Fail('Messsage doesn''t start with M')
			else begin
				AddLine(odd(ord(s[2])), StrToNum(s, 3), StrToNum(s, 6));
			end;
			ger.modified := true;
		end;

		procedure DoClick (where: point);
			var
				r, ro: rows0;
				c, co: cols0;
				vert, verto, old, valid: boolean;
		begin
			if ggame^.playstate = ps_playing then begin
				old := false;
				while StillDown do begin
					valid := false;
					GetMouse(where);
					if FindLine(where, r, c, vert) then
						if not ggame^.lines[vert, r, c] then
							valid := true;
					if old and (not valid or (r <> ro) or (c <> co)) then begin
						EraseLine(ggame^.globals, verto, ro, co);
						old := false;
					end;
					if valid and not old then begin
						DrawLine(ggame^.globals, vert, r, c);
						old := true;
						ro := r;
						co := c;
						verto := vert;
					end;
				end;
				if old then begin
					AddLine(verto, ro, co);
					SendMove(verto, ro, co);
					ger.modified := true;
				end;
			end;
		end;

		procedure InitGame;
			var
				i: integer;
				rct: rect;
				hdl: handle;
		begin
			ger.globals := Handle(NewPtr(sizeof(globalsRecord)));
			gglobals := globalsPeek(ger.globals);
			with rct, gglobals^.square_size, gglobals^ do begin
				for i := 0 to 3 do begin
					black[i * 2 + 0] := 255;
					black[i * 2 + 1] := 255;
					white[i * 2 + 0] := 0;
					white[i * 2 + 1] := 0;
					gray[i * 2 + 0] := $AA;
					gray[i * 2 + 1] := $55;
				end;
				hdl := GetResource('DITL', 128);
				if hdl = nil then begin
					Fail('GetResource DITL failed');
					SetRect(rct, 4, 4, 100, 100);
				end
				else
					BlockMove(ptr(longInt(hdl^) + 6), @rct, SizeOf(rect));
				rct.top := rct.top + 3;
				h := (right - left - 3 * width - 2 * inset - 2 * box_width - 2 * box_offset) div col_max;
				v := (bottom - top - 3 * width - 2 * inset - score_offset - score_height) div row_max;
				player_rect.left := left;
				player_rect.right := left + box_width;
				player_rect.top := top;
				left := left + box_width + box_offset;
				right := left + h * col_max + 3 * width + 2 * inset;
				winning_rect.left := right + box_offset;
				winning_rect.right := winning_rect.left + box_width;
				winning_rect.top := top;
				bottom := top + v * row_max + 3 * width + 2 * inset + score_offset + score_height + 2 * width;
				player_rect.bottom := bottom;
				winning_rect.bottom := bottom;
				score_rect.left := left;
				score_rect.right := right;
				score_rect.bottom := bottom;
				score_rect.top := bottom - score_height - width;
				bottom := bottom - score_offset - score_height - 2 * width;
				with score_rect, score_pos do begin
					h := (left + right) div 2;
					v := bottom - score_base;
				end;
				item_rect := rct;
				ger.int1 := left + right; { figure out why :-}
				ger.int2 := top + score_rect.bottom;
			end;
		end; {proc}

		procedure FinishGame;
		begin
			DisposPtr(POINTER(ger.globals));
			ger.globals := nil;
		end;

		procedure CommonInit;
			var
				k: integer;
				h: handle;
				r: rows0;
				c: cols0;
				rct: rect;
		begin
			GetDItem(gwindow, my_dialog_item, k, h, rct);
			SetDItem(gwindow, my_dialog_item, k, handle(@DrawGame), rct);
			SetWRefCon(gwindow, longInt(ghandle));
			ggame^.globals := gglobals;
		end;

		procedure UpdateControls;
		begin
			DrawPlayerBox(ggame);
		end;

		procedure RestartGame;
			var
				r: rows0;
				c: cols0;
		begin
			with ggame^ do begin
				for r := 0 to row_max do
					for c := 0 to col_max do begin
						lines[false, r, c] := false;
						lines[true, r, c] := false;
					end;
				for r := 1 to row_max do
					for c := 1 to col_max do
						board[r, c] := pt_None;
				move := first_player;
				boxes[pt_None] := row_max * col_max;
				boxes[pt_Black] := 0;
				boxes[pt_grey] := 0;
				playstate := ps_Playing;
				SetMyTurn;
				UpdateControls;
			end; {with}
		end;

		procedure GameNew;
			var
				r: rows0;
				c: cols0;
		begin
			HUnlock(ghandle);
			SetHandleSize(ghandle, SizeOf(gameRecord));
			HLock(ghandle);
			ggame := gamePeek(ghandle^);
			CommonInit;
			with ggame^ do begin
				connectionstate := cs_Local;
				mypiece := first_player;
				RestartGame;
			end; {with}
		end;

		procedure OldGame;
		begin
			CommonInit;
			ggame^.connectionstate := cs_Local;
{    ggame^.playstate := ps_Playing; ---- Don't do this, its a bug! }
			ger.myturn := true;
			UpdateControls;
		end;

		procedure Swap;
		begin
			with ggame^ do begin
				mypiece := Opposite(mypiece);
				SetMyTurn;
			end; {with}
			UpdateControls;
		end;

		procedure ConnectionLost;
		begin
			with ggame^ do begin
				connectionstate := cs_Local;
				SetMyTurn;
			end; {with}
		end;

		procedure ConnectionMade;
		begin
			with ggame^ do begin
				connectionstate := cs_Remote;
				SetMyTurn;
			end; {with}
		end;

		procedure Restart;
			var
				r: rect;
		begin
			RestartGame;
			ger.modified := true;
			r := gglobals^.item_rect;
			InsetRect(r, width, width);
			EraseRect(r);
			InvalRect(gwindow^.portRect);
		end;

		procedure MouseDown;
		begin
			DoClick(ger.where);
		end;

		procedure MessageReceived;
			var
				tmpstr: str255;
				x: longint;
				seq: longint;
				i, j: integer;
		begin
			DoMove(ger.message);
		end;

	begin
		gglobals := globalsPeek(ger.globals);
		ghandle := ger.game;
		if ghandle <> nil then begin
			HLock(ghandle);
			ggame := gamePeek(ghandle^);
		end;
		GetPort(gwindow);
		PenSize(width, width);
		case ger.event of
			ge_InitRuleBook: 
				InitGame;
			ge_FinishRuleBook: 
				FinishGame;
			ge_NewGame: 
				GameNew;
			ge_OldGame: 
				OldGame;
			ge_ConnectionLost: 
				ConnectionLost;
			ge_ConnectionMade: 
				ConnectionMade;
			ge_MessageReceived: 
				MessageReceived;
			ge_MouseDown: 
				MouseDown;
			ge_Swap: 
				Swap;
			ge_Restart: 
				Restart;
			otherwise
		end;
		PenNormal;
		if ghandle <> nil then
			HUnlock(ghandle);
	end;

end.