unit Othello;

{ Othello 1991 Rhys Hollow}
{This Rulebook and its source are both PovertyWare}

interface

	uses
		Debugs, DialogSubs, GameTypes, OthelloTypes, OthelloDraw, OthelloPlay;

	procedure Main (var gameevent: gameEventRecord);

implementation

	procedure Main (var gameevent: gameEventRecord);

		var
			g: gamePeek;
			gl: globalsHandle;

		procedure InitOthello;
			type
				rectPtr = ^rect;
				rectHandle = ^rectPtr;
			var
				i: integer;
				dr: rectHandle;
		begin
			gameevent.globals := NewHandle(sizeof(globalsRecord));
			dr := rectHandle(GetResource('DLOG', 1128));
			gameevent.int1 := dr^^.right - dr^^.left;
			gameevent.int2 := dr^^.bottom - dr^^.top;
			ReleaseResource(handle(dr));
			HLock(gameevent.globals);
			with globalsHandle(gameevent.globals)^^ do begin
				pieces[none] := GetIcon(1130);
				pieces[white] := GetIcon(1129);
				pieces[black] := GetIcon(1128);
				piecepicture[white] := GetIcon(1142);
				piecepicture[black] := GetIcon(1141);
				winpiece[black] := GetIcon(1143);
				winpiece[white] := GetIcon(1144);
				winposition[black] := GetPicture(1136);
				winposition[white] := GetPicture(1137);
				winposition[none] := GetPicture(1138);
				moveposition[white, white] := GetPicture(1134);
				moveposition[white, black] := GetPicture(1132);
				moveposition[black, white] := GetPicture(1135);
				moveposition[black, black] := GetPicture(1133);
				for i := 1 to 5 do begin
					flippieces[black, i] := GetIcon(1130 + i);
					flippieces[white, i] := GetIcon(1135 + i);
				end;
			end;
		end;

		procedure FinishOthello;
			var
				p: pieceType;
				i: integer;
		begin
			with gl^^ do begin
				for p := none to white do
					ReleaseResource(Handle(pieces[p]));
				for i := 1 to 5 do begin
					ReleaseResource(Handle(flippieces[black, i]));
					ReleaseResource(Handle(flippieces[white, i]));
				end;
				ReleaseResource(Handle(piecepicture[white]));
				ReleaseResource(Handle(piecepicture[black]));
				ReleaseResource(Handle(moveposition[white, white]));
				ReleaseResource(Handle(moveposition[white, black]));
				ReleaseResource(Handle(moveposition[black, white]));
				ReleaseResource(Handle(moveposition[black, black]));
				ReleaseResource(Handle(winpiece[black]));
				ReleaseResource(Handle(winpiece[white]));
				ReleaseResource(Handle(winposition[black]));
				ReleaseResource(Handle(winposition[white]));
				ReleaseResource(Handle(winposition[none]));
			end;
			DisposHandle(gameevent.globals);
			gameevent.globals := nil;
		end;

		procedure UpdateMenus;
		begin
			CheckItem(gl^^.gamemenu, showmovesCMD, g^.cheat);
		end;

		procedure ActivateOthello;
		begin
			g^.in_foreground := true;
			gl^^.gamemenu := GetMenu(gamemenuID);
			InsertMenu(gl^^.gamemenu, 0);
			DrawMenuBar;
		end;

		procedure DeactivateOthello;
		begin
			g^.in_foreground := false;
			DeleteMenu(gamemenuID);
			DisposeMenu(gl^^.gamemenu);
			DrawMenuBar;
		end;

		procedure CommonInit;
			var
				savport: GrafPtr;
				tr: rect;
		begin
			GetPort(savport);
			SetDHandle(savport, myuseritemID, Handle(@Draw));
			WindowPeek(savport)^.refcon := longint(gameevent.game);
			g^.globals := gl;
			GetDRect(DialogPtr(savport), myuseritemID, tr);
			g^.globals^^.board := tr.topleft;
			g^.gamestate := gs_Local;
			g^.playstate := ps_Playing;
		end;

		procedure NewGame;
			var
				i, j: integer;
				dummy: boolean;
				hstate: SignedByte;
		begin
			SetHandleSize(Handle(gameevent.game), sizeof(gameRecord));
			hstate := HGetState(Handle(gameevent.game));
			HLock(Handle(gameevent.game));
			g := gamePeek(Handle(gameevent.game)^);
			CommonInit;
			with g^ do begin
				for i := -1 to 8 do
					for j := -1 to 8 do
						board[i, j] := none;
				board[3, 3] := black;
				board[3, 4] := white;
				board[4, 3] := white;
				board[4, 4] := black;
				move := white;
				cheat := false;
				mypiece := white;
				inrgn := NewRgn;
			end;
			dummy := CanMove(g);
		end;

		procedure OldGame;
			var
				dummy: boolean;
		begin
			CommonInit;
			g^.gamestate := gs_Local;
			g^.inrgn := NewRgn;
			dummy := CanMove(g);
			UpdateScore(g);
			UpdatePieces(g);
			gameevent.myturn := true;
			NextMove(g);
		end;

		procedure Close;
		begin
			DisposeRgn(g^.inrgn);
		end;

		procedure Swap;
			var
				tr: rect;
				savport: GrafPtr;
		begin
			g^.mypiece := Opposite(g^.mypiece);
			UpdatePieces(g);
			UpdateScore(g);
			gameevent.myturn := (g^.mypiece = g^.move) or (g^.gamestate = gs_Local);
		end;

		procedure Restart;
			var
				i, j: integer;
				dummy: boolean;
		begin
			if g^.cheat then
				InvertRgn(g^.inrgn);
			for j := 0 to 7 do begin
				for i := 0 to 7 do begin
					if (i = 3) and (j = 3) then
						ChangePiece(g, i, j, black)
					else if (i = 3) and (j = 4) then
						ChangePiece(g, i, j, white)
					else if (i = 4) and (j = 3) then
						ChangePiece(g, i, j, white)
					else if (i = 4) and (j = 4) then
						ChangePiece(g, i, j, black)
					else
						ChangePiece(g, i, j, none);
				end;
			end;
			g^.move := white;
			gameevent.modified := false;
			g^.playstate := ps_Playing;
			dummy := CanMove(g);
			UpdateScore(g);
			UpdatePieces(g);
		end;

		procedure ConnectionMade;
			var
				dummy: boolean;
		begin
			g^.gamestate := gs_Remote;
			gameevent.myturn := g^.move = g^.mypiece;
			dummy := CanMove(g);
		end;

		procedure ConnectionLost;
		begin
			g^.gamestate := gs_Local;
			gameevent.myturn := true;
		end;


		procedure Menu;
		begin
			if gameevent.int2 = showmovesCMD then begin
				InvertRgn(g^.inrgn);
				g^.cheat := not g^.cheat;
			end;
		end;

		procedure MouseDown;
			var
				tmppt: point;
				gridx, gridy: integer;
				tr: rect;
				savport: GrafPtr;
				beep: boolean;
		begin
			if g^.playstate = ps_Playing then begin
				beep := true;
				GetPort(savport);
				GetDRect(savport, myuseritemID, tr);
				InsetRect(tr, 1, 1);
				if PtInRect(gameevent.where, tr) then begin
					GetRect(g, gameevent.where, gridx, gridy);
					if ValidMove(g, gridx, gridy) then begin
						beep := false;
						if TrackMove(g, gridx, gridy) then begin
							PlacePiece(g, gridx, gridy);
							gameevent.modified := true;
							g^.move := Opposite(g^.move);
							if g^.move <> g^.mypiece then begin
								gameevent.event := ge_SendMessage;
								gameevent.message := concat(chr(gridx + ord('0')), chr(gridy + ord('0')));
							end;
							NextMove(g);
							gameevent.myturn := (g^.move = g^.mypiece) or (g^.gamestate = gs_Local);
						end;
					end;
					if beep then
						SysBeep(10);
				end;
			end;
		end;

		procedure MessageReceived;
			var
				s: str255;
				x, y: integer;
		begin
			s := gameevent.message;
			x := ord(s[1]) - ord('0');
			y := ord(s[2]) - ord('0');
			PlacePiece(g, x, y);
			gameevent.modified := true;
			g^.move := Opposite(g^.move);
			NextMove(g);
			gameevent.myturn := (g^.move = g^.mypiece) or (g^.gamestate = gs_Local);
		end;

		var
			hstate: SignedByte;

	begin
		if (gameevent.game <> nil) and (gameevent.event <> ge_NewGame) then begin
			hstate := HGetState(Handle(gameevent.game));
			HLock(Handle(gameevent.game));
			g := gamePeek(Handle(gameevent.game)^);
		end;
		if gameevent.globals <> nil then begin
			HLock(gameevent.globals);
			gl := globalsHandle(gameevent.globals);
		end;
		case gameevent.event of
			ge_InitRuleBook: 
				InitOthello;
			ge_FinishRuleBook: 
				FinishOthello;
			ge_UpdateMenus: 
				UpdateMenus;
			ge_Activate: 
				ActivateOthello;
			ge_Deactivate: 
				DeactivateOthello;
			ge_NewGame: 
				NewGame;
			ge_OldGame: 
				OldGame;
			ge_Close: 
				Close;
			ge_ConnectionLost: 
				ConnectionLost;
			ge_ConnectionMade: 
				ConnectionMade;
			ge_MessageReceived: 
				MessageReceived;
			ge_MouseDown: 
				MouseDown;
			ge_KeyDown: 
				;
			ge_Menu: 
				Menu;
			ge_Swap: 
				Swap;
			ge_Restart: 
				Restart;
			otherwise
		end;
		if (gameevent.game <> nil) and (gameevent.event <> ge_NewGame) then
			HSetState(Handle(gameevent.game), hstate);
		if (gameevent.globals <> nil) then
			HUnlock(gameevent.globals);
	end;
end.